#####################################################
# leonov - The Launchpad Desktop Client
# Copyright (C) 2008 Stephan Hermann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#####################################################
import xdg.BaseDirectory as basedir
try:
    #python >= 2.5 ships sqlite module
    import sqlite3 as sqlite
except:
    from pysqlite2 import dbapi2 as sqlite
import logging
import os.path

class LPDB():
    
    def __init__(self):
        self.dbname="launchpad.sql.db"
        self.savepath=os.path.join(basedir.save_config_path("leonov"),self.dbname)
        # check if that exists:
        self.cursor=None
        if not os.path.exists(self.savepath):
            self.open_db()
            # create the tables:
            self.create_tables()
            self.db_connection.close()
            self.cursor=None
            
    def open_db(self):
        self.db_connection=sqlite.connect(self.savepath,isolation_level=None)
        if not self.cursor:
            self.cursor=self.db_connection.cursor()


    def create_tables(self):
        # Usertable
        self.create_table("lp_users","user_id INTEGER PRIMARY KEY AUTOINCREMENT,realname varchar(100),lp_id varchar(50)")
        
        # Project table
        self.create_table("lp_projects","project_id INTEGER PRIMARY KEY AUTOINCREMENT,name varchar(100),summary text, url varchar(255),registrar INTEGER,license_reviewed INTEGER, registered_date varchar(30)")
        
    def create_table(self,table_name,table_data):
        # Usertable
        self.cursor.execute( "CREATE TABLE %s ( %s )" % (table_name,table_data))
        
    def select_from_table(self,table,select_string,filter_string=""):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB::select_from_table: table %s select_string: %s filter_string: %s",table,select_string,filter_string)
        if table:
            if filter_string=="":
                self.cursor.execute("SELECT %s FROM %s" % (select_string,table))
            else:
                self.cursor.execute("SELECT %s FROM %s WHERE %s" % (select_string,table,filter_string))
        return self.cursor.fetchall()
    
    def commit(self):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB::Commit commited")
        self.db_connection.commit()
    
    def close(self):
        self.db_connection.close()
        
class LPDB_Users():
    def __init__(self,lpdb=None):
        self.lpdb=lpdb
        if not self.lpdb:
            self.lpdb=LPDB()
        self.lpdb.open_db()
        
    def select(self,select_filter=None,search_filter=None):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Users::select")
        if not select_filter:
            select_filter="*"
        if not search_filter:
            search_filter=""
        return self.lpdb.select_from_table("lp_users",select_filter,search_filter)
        
    def add(self,lp_user):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Users::add")
        # (realname,lp_id)
        self.lpdb.cursor.execute("INSERT INTO lp_users values(null,?,?)",(lp_user.realname,str(lp_user)))
        return self.lpdb.cursor.lastrowid
        
    def add_many(self, users_iter):
        self.lpdb.cursor.executemany("INSERT INTO lp_users values(null,?,?)", users_iter)
        return self.lpdb.cursor.lastrowid #what is rerally needed here?
        
    def commit(self):
        self.lpdb.commit()
        
        
        
class LPDB_Projects():
    def __init__(self,lpdb=None):
        self.lpdb=lpdb
        if not self.lpdb:
            self.lpdb=LPDB()
        self.lpdb.open_db()
        
    def select(self,select_filter=None,search_filter=None):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Projects::select")
        if not select_filter:
            select_filter="*"
        if not search_filter:
            search_filter=""
        return self.lpdb.select_from_table("lp_projects",select_filter,search_filter)
    
    def add(self,project,user_id):
        logging.getLogger("leonov.backend.sqlinterface").debug("LPDB_Projects::add name: %s , user_id: %d",project.name,user_id)
        # (name,summary,url,registrar,license_reviewed,registered_date)        
        self.lpdb.cursor.execute("INSERT INTO lp_projects  values(null,?,?,?,?,?,?)", (project.name,project.summary,project.url,user_id,int(project.license_reviewed),project.registered))
        return self.lpdb.cursor.lastrowid
        
    def add_many(self, project_iter):
        self.lpdb.cursor.executemany("INSERT INTO lp_projects values(null,?,?,?,?,?,?)", project_iter)
        return self.lpdb.cursor.lastrowid #what is rerally needed here?
    
    def commit(self):
        self.lpdb.commit()

    
